==> GotoBLAS-1.00.tar.gz: GotoBLAS/level2/hemv/zhemv_k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifndef SYMV_P
#define SYMV_P (DTB_ENTRIES / 2)
#endif

int CNAME(long m, FLOAT alpha_r, FLOAT alpha_i, 
	 FLOAT *a, long lda, FLOAT *x, long incx, FLOAT *y, long incy, void *buffer){

  long is;
  long min_i;

  FLOAT *subbuffer = buffer + SUB_BUFFER_OFFSET;  /* Buffer */

  for(is = 0; is < m; is += SYMV_P){

    min_i = MIN(SYMV_P, m - is);
      
#ifndef LOWER
    if (is >= SYMV_P){
      ZGEMV_C(is, min_i, 0, alpha_r, alpha_i, 
	      a + is * lda * 2,  lda,
	      x,                 incx,
	      y + is * incy * 2, incy, buffer);

      ZGEMV_N(is, min_i, 0, alpha_r, alpha_i,
	      a + is * lda * 2,  lda,
	      x + is * incx * 2, incx,
	      y,             incy, buffer);
    }
#endif

#ifdef LOWER
    ZHEMCOPY_L(min_i, a + (is + is * lda) * 2, lda, subbuffer);
#else
    ZHEMCOPY_U(min_i, a + (is + is * lda) * 2, lda, subbuffer);
#endif
    
    ZGEMV_N(min_i, min_i, 0, alpha_r, alpha_i, 
	    subbuffer, min_i,
	    x + is * incx * 2, incx, 
	    y + is * incy * 2, incy, buffer);
    
#ifdef LOWER
    if (m - is - SYMV_P > 0){
      ZGEMV_C(m - is - SYMV_P, min_i, 0, alpha_r, alpha_i,
	      a + ((is + SYMV_P) + is * lda) * 2, lda,
	      x + (is + SYMV_P) * incx * 2,  incx,
	      y + is * incy * 2,         incy, buffer);

      ZGEMV_N(m - is - SYMV_P, min_i, 0, alpha_r, alpha_i,
	     a + ((is + SYMV_P) + is * lda) * 2, lda,
	     x + is * incx * 2,         incx,
	     y + (is + SYMV_P) * incy * 2,  incy, buffer);
    }
#endif

  } /* end of is */

  return 0;
}

==> GotoBLAS-1.10.tar.gz: GotoBLAS/level2/hemv/zhemv_k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

int CNAME(long m, FLOAT alpha_r, FLOAT alpha_i, 
	 FLOAT *a, long lda, FLOAT *x, long incx, FLOAT *y, long incy, void *buffer){

  long is, min_i;
  FLOAT *X = x;
  FLOAT *Y = y;
  FLOAT *symbuffer  = buffer;
  FLOAT *gemvbuffer = (FLOAT *)(((long)buffer + SYMV_P * SYMV_P * sizeof(FLOAT) * 2 + 4095) & ~4095);
  FLOAT *bufferY    = gemvbuffer;
  FLOAT *bufferX    = gemvbuffer;

  if (incy != 1) {
    Y = bufferY;
    bufferX    = (FLOAT *)(((long)bufferY + m * sizeof(FLOAT) * 2 + 4095) & ~4095);
    gemvbuffer = bufferX;
    ZCOPY_K(m, y, incy, Y, 1);
  }

  if (incx != 1) {
    X = bufferX;
    gemvbuffer = (FLOAT *)(((long)bufferX + m * sizeof(FLOAT) * 2 + 4095) & ~4095);
    ZCOPY_K(m, x, incx, X, 1);
  }

  for(is = 0; is < m; is += SYMV_P){

    min_i = MIN(SYMV_P, m - is);
      
#ifndef LOWER
    if (is >= SYMV_P){
      ZGEMV_C(is, min_i, 0, alpha_r, alpha_i, 
	      a + is * lda * 2,  lda,
	      X,          1,
	      Y + is * 2, 1, gemvbuffer);

      ZGEMV_N(is, min_i, 0, alpha_r, alpha_i,
	      a + is * lda * 2,  lda,
	      X + is * 2, 1,
	      Y,          1, gemvbuffer);
    }
#endif

#ifdef LOWER
    ZHEMCOPY_L(min_i, a + (is + is * lda) * 2, lda, symbuffer);
#else
    ZHEMCOPY_U(min_i, a + (is + is * lda) * 2, lda, symbuffer);
#endif
    
    ZGEMV_N(min_i, min_i, 0, alpha_r, alpha_i, 
	    symbuffer, min_i,
	    X + is * 2, 1, 
	    Y + is * 2, 1, gemvbuffer);
    
#ifdef LOWER
    if (m - is - SYMV_P > 0){
      ZGEMV_C(m - is - SYMV_P, min_i, 0, alpha_r, alpha_i,
	      a + ((is + SYMV_P) + is * lda) * 2, lda,
	      X + (is + SYMV_P) * 2, 1,
	      Y +  is           * 2, 1, gemvbuffer);

      ZGEMV_N(m - is - SYMV_P, min_i, 0, alpha_r, alpha_i,
	      a + ((is + SYMV_P) + is * lda) * 2, lda,
	      X +  is           * 2,  1,
	      Y + (is + SYMV_P) * 2,  1, gemvbuffer);
    }
#endif

  } /* end of is */

  if (incy != 1) {
    ZCOPY_K(m, Y, 1, y, incy);
  }

  return 0;
}
